use std::collections::VecDeque;

use anyhow::Context;
use subprocess::{Communicator, Popen, PopenConfig, Redirection};

pub struct RtlSdr {
    comms: Communicator,
    out_buf: VecDeque<u8>,
    err_buf: Vec<u8>,
}

impl RtlSdr {
    pub fn new() -> anyhow::Result<Self> {
        let mut p = Popen::create(
            &[
                "rtl_fm",
                "-f",
                "430500000",
                "-M",
                "raw",
                "-F9",
                "-p",
                "0",
                "-d",
                "0",
                "-s",
                "48000",
            ],
            PopenConfig {
                stdout: Redirection::Pipe,
                stderr: Redirection::Pipe,
                detached: true,
                ..Default::default()
            },
        )
        .context("Error starting rtl-fm")?;

        let comms = p.communicate_start(None).limit_size(1024);

        Ok(Self {
            comms,
            out_buf: VecDeque::new(),
            err_buf: vec![],
        })
    }
}

impl Iterator for RtlSdr {
    type Item = u8;

    fn next(&mut self) -> Option<Self::Item> {
        if let Some(b) = self.out_buf.pop_front() {
            return Some(b);
        }

        let (out, err) = self.comms.read().ok()?;
        let out = out.expect("Missing stdout");
        let err = err.expect("Missing stderr");

        self.out_buf.extend(out.iter());
        for e in err {
            if e == b'\n' {
                handle_err(&self.err_buf);
                self.err_buf.clear();
            } else {
                self.err_buf.push(e);
            }
        }

        self.out_buf.pop_front()
    }
}

fn handle_err(err: &[u8]) {
    eprintln!("[rtl_fm] {}", String::from_utf8_lossy(err));
}
